import java.io.*;
import java.security.*;

public class Signer {
    private static final int MODE_SIGN=0;
    private static final int MODE_VERIFY=1;

    private static void printUsage()throws Exception {
	System.out.println(
          "Usage: Signer -sign keystore storepass keypass alias messagefile signaturefile");
	System.out.println(
          "Usage: Signer -verify keystore storepass alias messagefile signaturefile");
    }
  public static void main(String[] args) throws Exception {

    String options = args[0];
    String keystorefile = args[1];
    String storepass = args[2];
    String keypass = null;
    String alias = null;
    String messagefile = null;
    String signaturefile = null;
    int mode;

    if (args.length == 7 && options.equals("-sign")) {
	keypass = args[3];
	alias = args[4];
	messagefile = args[5];
	signaturefile = args[6];
	mode=MODE_SIGN;
    } else  if (args.length == 6 && options.equals("-verify")) {
	alias = args[3];
	messagefile = args[4];
	signaturefile = args[5];
	mode=MODE_VERIFY;
    } else {
	printUsage();
	return;
    }

    Signature signature = Signature.getInstance("DSA");

    KeyStore keystore = KeyStore.getInstance("JKS");
    keystore.load(new FileInputStream(keystorefile), storepass.toCharArray());

    if (mode==MODE_SIGN)
      signature.initSign((PrivateKey)keystore.getKey(alias, keypass.toCharArray()));
    else
      signature.initVerify(keystore.getCertificate(alias).getPublicKey());

    FileInputStream in = new FileInputStream(messagefile);
    byte[] buffer = new byte[8192];
    int length;
    while ((length = in.read(buffer)) != -1)
      signature.update(buffer, 0, length);
    in.close();

    if (mode==MODE_SIGN) {
      FileOutputStream out = new FileOutputStream(signaturefile);
      byte[] sig = signature.sign();
      out.write(sig);
      out.close();
    } else {
      FileInputStream sigIn = new FileInputStream(signaturefile);
      byte[] sigToVerify = new byte[sigIn.available()];
      sigIn.read(sigToVerify);
      sigIn.close();
      if (signature.verify(sigToVerify))
        System.out.println("The signature is correct.");
      else
        System.out.println("The signature is CORRUPTED !!!");
    }
  }
}
